import 'package:args/args.dart';
import 'package:args/command_runner.dart';
import 'package:lokalise_cmd/src/command/flutter_lokalise_dart_command_runner.dart';

import 'arg_results_extension.dart';

abstract class FlutterLokaliseCommand<T> extends Command<T> {
  FlutterLokaliseCommandRunner get commandRunner =>
      (runner as FlutterLokaliseCommandRunner);
}

class FlutterLokaliseArgResults {
  final String? workingDirectory;
  final String? apiToken;
  final String? projectId;

  FlutterLokaliseArgResults.fromArgResults(
    ArgResults results, {
    String? fallbackApiToken,
    String? fallbackProjectId,
  })  : workingDirectory = results.getNullable("working-directory"),
        apiToken = results.get("api-token", orElse: fallbackApiToken),
        projectId = results.get("project-id", orElse: fallbackProjectId);

  static void addOptions(ArgParser argParser) {
    argParser.addOption(
      "working-directory",
      abbr: "d",
      help: "working directory path",
    );
    argParser.addOption(
      "api-token",
      abbr: "t",
      help: "Lokalise API token",
    );
    argParser.addOption(
      "project-id",
      abbr: "i",
      help: "Lokalise project id",
    );
  }
}
