import 'dart:async';
import 'dart:io';

import 'package:args/args.dart';
import 'package:args/command_runner.dart';
import 'package:lokalise_cmd/src/lokalise_config/lokalise_config.dart';
import 'package:logging/logging.dart';

import 'download_command.dart';
import 'flutter_lokalise_command.dart';

class FlutterLokaliseCommandRunner extends CommandRunner<Null> {
  final Logger _logger;
  FlutterLokaliseArgResults? _flutterLokaliseArgResults;
  LokaliseConfig? _lokaliseConfig;

  FlutterLokaliseArgResults? get flutterLokaliseArgResults =>
      _flutterLokaliseArgResults;

  LokaliseConfig get lokaliseConfig => _lokaliseConfig!;

  FlutterLokaliseCommandRunner.withDefaultCommands()
      : this(commands: [
          DownloadCommand(),
        ]);

  FlutterLokaliseCommandRunner({
    required List<Command<Null>> commands,
    Logger? logger,
  })  : _logger = logger ?? Logger.root,
        super(
          "lokalise_cmd",
          "A tool to download translation files from Lokalise and convert them to .json files.",
        ) {
    FlutterLokaliseArgResults.addOptions(argParser);
    commands.forEach((it) => addCommand(it));
  }

  @override
  Future<Null> runCommand(ArgResults topLevelResults) {
    final workingDirectory = _determineWorkingDirectory(topLevelResults);
    _lokaliseConfig = LokaliseConfig.fromPubspecYamlString(
        File("$workingDirectory/lokalise.yaml").readAsStringSync());
    _flutterLokaliseArgResults = FlutterLokaliseArgResults.fromArgResults(
      topLevelResults,
      fallbackApiToken: _lokaliseConfig!.apiToken,
      fallbackProjectId: _lokaliseConfig!.projectId,
    );
    _configureLogger(_flutterLokaliseArgResults!);
    return super.runCommand(topLevelResults);
  }

  void _configureLogger(FlutterLokaliseArgResults results) {
    _logger.level = Level.ALL;
    _logger.onRecord.listen((LogRecord logRecord) {
      if (logRecord.level >= Level.SEVERE) {
        stderr.writeln(logRecord);
      } else {
        stdout.writeln(logRecord);
      }
    });
  }

  String _determineWorkingDirectory(ArgResults results) =>
      FlutterLokaliseArgResults.fromArgResults(results).workingDirectory ??
      Directory.current.path;
}
